// Vertex shader for SH lighting of Grace Cathedral

uniform mat4 mvpMatrix;
attribute vec4 vPosition;

varying vec3  diffuseColor;

const float c1 = 0.429043;
const float c2 = 0.511664;
const float c3 = 0.743125;
const float c4 = 0.886227;
const float c5 = 0.247708;

// Constants for Grace Cathedral lighting
const vec3 L00  = vec3( 0.79,  0.44,  0.54);
const vec3 L1_1 = vec3( 0.39,  0.35,  0.60);
const vec3 L10  = vec3(-0.34, -0.18, -0.27);
const vec3 L11  = vec3(-0.29, -0.06,  0.01);
const vec3 L2_2 = vec3(-0.11, -0.05, -0.12);
const vec3 L2_1 = vec3(-0.26, -0.22, -0.47);
const vec3 L20  = vec3(-0.16, -0.09, -0.15);
const vec3 L21  = vec3( 0.56,  0.21,  0.14);
const vec3 L22  = vec3( 0.21, -0.05, -0.30);


void main(void)
{
   vec3 N;
   N = vPosition.xyz;       //normal of  a point on sphere
   N = normalize( N );      //normalize

   float x = N.x, y = N.y, z = N.z;

   diffuseColor    = c1 * L22 * (x*x - y*y) +
                     c3 * L20 * z*z +
                     c4 * L00 - c5 * L20 +
                     2.0 * c1 * (L2_2 * x*y + L21 * x*z + L2_1 * y*z) +
                     2.0 * c2 * (L11  * x + L1_1 * y + L10  * z);

     gl_Position = mvpMatrix * vPosition;
}
