  // spheribl.vert: Source code of vertex shader
   uniform mat4 mvpMatrix;
   attribute vec4 vPosition;
   uniform vec4 eyePosition;
   uniform vec4 lightPosition;
   uniform vec4 lightDiffuse;
   uniform vec4 materialColor;
   varying vec4 lightReflection;
   varying vec3 envReflectDir;   //environment reflection direction

   void main()
   {
      vec3 N = vPosition.xyz;    //normal of  a point on sphere
                                 // (= vPosition.xyz - origin.xyz)
      vec3 L = lightPosition.xyz - vPosition.xyz;
      vec3 V = eyePosition.xyz - vPosition.xyz;

      N = normalize(N);         //unit normal direction
      L = normalize(L);         //unit light source direction
      V = normalize(V);         //unit view vector

      // using a simplified lighting model
      float F = max(0.0, dot(N, L));
      lightReflection =  F * materialColor *  lightDiffuse;

      //environment reflection direction
      envReflectDir = 2.0 *  dot ( N, V) * N - V;

      gl_Position = mvpMatrix * vPosition;
  }

