// Vertex Shader of uberlight-like model 
uniform mat4 mvpMatrix;
attribute vec4 vPosition;
uniform vec4 lightPos;      // light position in world coordinates
uniform vec4 eyePos;        // view position in world space

varying vec3 vPosLC;        // vertex position in light coordinates
varying vec3 normLC;        // normal in light coordinates
varying vec3 eyeLC;         // view point in light coordinates

void main()
{
    vec3 L = normalize ( lightPos.xyz );
    float d = sqrt (L.y*L.y + L.z*L.z);
    mat3 Rx, Ry, R;    // rotation matrices

    Rx = mat3 ( 1, 0,     0,          //1st column
                0, L.z/d, L.y/d,      //2nd column
                0, -L.y/d, L.z/d );   //3rd column
    Ry = mat3 ( d,  0, L.x,           //1st column
                0,   1, 0,            //2nd column
                -L.x, 0, d );         //3rd column

    R = Ry * Rx;                  //the composite matrix
    vPosLC = R * (vPosition.xyz - lightPos.xyz); 
    eyeLC  = R * (eyePos.xyz - lightPos.xyz);
    normLC = R * vPosition.xyz;

    gl_Position = mvpMatrix * vPosition;
}
